local DatabaseTool = require "DatabaseTool"

EngineFFI = nil

if not EngineFFI then
	--package.loaded["EngineFFI"] = nil
	local _EngineFFI = require "EngineFFI"
	if _EngineFFI then
		_EngineFFI.Load()
		EngineFFI = _EngineFFI
	end
end

local function StringBuilder()
	local sb = {
		content = {},
		append = function( self, str )
			self.content[ table.maxn( self.content ) + 1 ] = str
		end,
		appendFormat = function( self, fmt, ... )
			self:append( string.format( fmt, ... ) )
		end,
		toString = function( self, sep )
			return table.concat( self.content, sep or '\n' )
		end,
		clear = function( self )
			self.content = {}
		end,
	}
	return sb
end

local function WaitForScene()
	WorldState = WorldState_mt.__static_instance
	local yield = yield
	local scene = nil
	while true do
		scene = WorldState.activeScene
		if scene then
			break
		end
		yield()
	end
	return scene
end

local function WaitForLocalPlayer()
	local scene = WaitForScene()
	local lp = nil
	local yield = yield
	while true do
		lp = scene:GetLocalPlayer()
		if lp then
			break
		end
		yield()
	end
	return lp
end

local function SceneCall( func )
	local WorldState = WorldState_mt.__static_instance
	local scene = WorldState.activeScene
	if not scene then
		for i = 1, 1000 do
			scene = WorldState.activeScene
			if scene then
				break
			end
			yield()
		end
	end
	if scene then
		print( "SceneCall: name: "..scene.name )
		func( scene )
	end
end

local function PlanAnimEventCall( animationEventDispatcher, eventName, func )
	if animationEventDispatcher ~= nil then
		local callback = nil
		local adder = animationEventDispatcher[ "AddEventHandler_"..eventName ]
		if adder then
			callback = adder(
				animationEventDispatcher,
				function()
					local remover = animationEventDispatcher[ "RemoveEventHandler_"..eventName ]
					if remover then
						remover( animationEventDispatcher, callback )
					end
					func()
				end
			)
		end
	end
end

return {
	LoadFFI = function()
		if not EngineFFI then
			EngineFFI = require "EngineFFI"
			EngineFFI.Load()
		end
		return EngineFFI
	end,

	ReloadFFI = function()
		package.loaded["EngineFFI"] = nil
		EngineFFI = require "EngineFFI"
		EngineFFI.Load()
		return EngineFFI
	end,

	WaitForScene = WaitForScene,
	WaitForLocalPlayer = WaitForLocalPlayer,
	PlanAnimEventCall = PlanAnimEventCall,
	SceneCall = SceneCall,
	StringBuilder = StringBuilder
}
