
local function Load()
	if not LuaTinker then
		print( "LuaTinker has't been initialized!" )
		return
	end

	LuaTinker.Using( 'System' )
	LuaTinker.Using( 'Common' )
	LuaTinker.Using( 'UniScene' )
	LuaTinker.Using( 'GwScene' )
	LuaTinker.Using( 'UnityEngine' )
	LuaTinker.Using( 'NewMission' )
	LuaTinker.Using( 'GwScene.Cutscene' )

	if USE_NATIVE_LUA then
		LuaTinker.Using( 'NativeLua' )
	else
		LuaTinker.Using( 'UniLuax' )
	end

	ScriptManager_mt = LuaTinker.DefineClass( "ScriptManager" )
		:sprop( "ScriptManager sharedInstance" )
		:method( "int RunFile( String filePath )" )
		:method( "bool IsAlive( int id )" )
		:method( "bool Close( int id )" )
	:EndDefine()

	LuaFunctor_mt = LuaTinker.DefineClass( "LuaFunctor" )
	:EndDefine()

	mission_SigGroup_mt = LuaTinker.DefineClass( "Game.mission_SigGroup" )
		:method( "System.Object AddEvent_AddNewAcceptTask( LuaFunctor function )" )
		:method( "bool RemoveEvent_AddNewAcceptTask( System.Object function )" )
	:EndDefine()

	Sig_mt = LuaTinker.DefineClass( "Game.Sig" )
		:sfield( "Game.mission_SigGroup mission" )
	:EndDefine()

	GameObject_mt = LuaTinker.DefineClass( "UnityEngine.GameObject" )
		:ctor( "String" )
		:prop( "String name" )
		:method( "void SetActive( bool )" )
	:EndDefine()

	Vector2_mt = LuaTinker.DefineClass( "UnityEngine.Vector2" )
		:ctor( "float, float" )
		:field( "float x" )
		:field( "float y" )
	:EndDefine()

	Vector3_mt = LuaTinker.DefineClass( "UnityEngine.Vector3" )
		:ctor( "float, float" )
		:ctor( "float, float, float" )
		:field( "float x" )
		:field( "float y" )
		:field( "float z" )
	:EndDefine()

	Quaternion_mt = LuaTinker.DefineClass( "UnityEngine.Quaternion" )
		:ctor( "float, float, float, float" )
		:field( "float x" )
		:field( "float y" )
		:field( "float z" )
		:field( "float w" )
	:EndDefine()

	Object_mt = LuaTinker.DefineClass( "UnityEngine.Object" )
		:prop( "String name" )
	:EndDefine()

	Component_mt = LuaTinker.DefineClass( "UnityEngine.Component" )
		:base( "UnityEngine.Object" )
		:prop( "Transform transform" )
		:prop( "GameObject gameObject" )
		:prop( "bool active" )
		:method( "Component GetComponent( string )" )
		:method( "void SendMessage( string )" )
	:EndDefine()

	Transform_mt = LuaTinker.DefineClass( "UnityEngine.Transform" )
		:base( "UnityEngine.Component" )
		:prop( "GameObject gameObject" )
		:prop( "int childCount" )
		:prop( "Vector3 eulerAngles" )
		:prop( "Vector3 forward" )
		:prop( "bool hasChanged" )
		:prop( "Vector3 localEulerAngles" )
		:prop( "Vector3 localPosition" )
		:prop( "Quaternion localRotation" )
		:prop( "Vector3 localScale" )
		:prop( "Vector3 lossyScale" )
		:prop( "Transform parent" )
		:prop( "Vector3 position" )
		:prop( "Vector3 right" )
		:prop( "Transform root" )
		:prop( "Quaternion rotation" )
		:prop( "Vector3 up" )
		:method( "void DetachChildren()" )
		:method( "Transform Find( string a )" )
		:method( "Transform FindChild( string )" )
		:method( "Transform GetChild( int )" )
		:method( "void LookAt( Transform )" )
	:EndDefine()

	Behaviour_mt = LuaTinker.DefineClass( "UnityEngine.Behaviour" )
		:base( "UnityEngine.Component" )
		:prop( "bool enabled" )
	:EndDefine()

	MonoBehaviour_mt = LuaTinker.DefineClass( "UnityEngine.MonoBehaviour" )
		:base( "UnityEngine.Behaviour" )
	:EndDefine()

	LuaBehaviour_mt = LuaTinker.DefineClass( "Common.LuaBehaviour" )
		:base( "UnityEngine.MonoBehaviour" )
		:prop( "String sourcePath" )
	:EndDefine()

	Camera_mt = LuaTinker.DefineClass( "UnityEngine.Camera" )
		:base( "UnityEngine.Behaviour" )
	:EndDefine()

	AnimEventDispatcher_mt = LuaTinker.DefineClass( "AnimEventDispatcher" )
		:method( "void ClearAllSigSlots()" )
		:method( "object AddEventHandler_ActionBegin( LuaFunctor )" )
		:method( "bool RemoveEventHandler_ActionBegin( object function )" )
		:method( "object AddEventHandler_ActionEnd( LuaFunctor )" )
		:method( "bool RemoveEventHandler_ActionEnd( object function )" )
		:method( "object AddEventHandler_Missile( LuaFunctor )" )
		:method( "bool RemoveEventHandler_Missile( object function )" )
		:method( "object AddEventHandler_Hit( LuaFunctor )" )
		:method( "bool RemoveEventHandler_Hit( object function )" )
		:method( "object AddEventHandler_BladeBegin( LuaFunctor )" )
		:method( "bool RemoveEventHandler_BladeBegin( object function )" )
		:method( "object AddEventHandler_ActionBegin( LuaFunctor )" )
		:method( "bool RemoveEventHandler_ActionBegin( object function )" )
		:method( "object AddEventHandler_ActionBegin( LuaFunctor )" )
		:method( "bool RemoveEventHandler_ActionBegin( object function )" )
	:EndDefine()

	Entity_mt = LuaTinker.DefineClass( "Entity" )
		:prop( "Vector3 position" )
		:prop( "Vector2 position2d" )
		:prop( "UInt64 code" )
		:prop( "float facing" )
		:prop( "int layer" )
		:prop( "GameObject gameObject" )
		:prop( "Transform transform" )
		:prop( "int layer" )
		:prop( "AnimEventDispatcher animationEventDispatcher" )
		:method( "void RayCastToGround( Vector3 )" )
		:method( "void RayCastToGroundXOZ( Vector2 )" )
		:method( "void SetPosition( Vector3 )" )
		:method( "void SetTargetPosition( Vector2 )", "SetTargetPosition2d" )
		:method( "void SetTargetPosition( Vector3 )" )
	:EndDefine()

	LuaTinker.DefineEnum( "Camp" )
	LuaTinker.DefineEnum( "Career" )

	local Character_mt = LuaTinker.DefineClass( "Character" )
		:base( "Entity" )
		:method( "bool RunASMCmd( String )" )
	:EndDefine()

	Player_mt = LuaTinker.DefineClass( "Player" )
		:base( "Character" )
		:method( "int GetProp_Level()" )
		:method( "int GetProp_HpPercent()" )
		:method( "String GetProp_Name()" )
		:method( "String GetProp_RoleID()" )
		:method( "Career GetProp_Career()" )
	:EndDefine()

	LocalPlayer_mt = LuaTinker.DefineClass( "LocalPlayer" )
		:base( "Player" )
		:method( "void EnableUserInput( bool )" )
		:method( "void Freeze( float )" )
		:method( "void UnFreeze()" )
		:method( "bool IsInDeadZone()" )
	:EndDefine()

	RemotePlayer_mt = LuaTinker.DefineClass( "RemotePlayer" )
		:base( "Player" )
	:EndDefine()

	CameraControl_mt = LuaTinker.DefineClass( "CameraControl" )
	:EndDefine()

	BaseScene_mt = LuaTinker.DefineClass( "BaseScene" )
		:prop( "String name" )
		:prop( "Camera camera" )
		:method( "void EnableSceneObjects( bool )" )
		:smethod( "Vector3 GetTerrainPosition( float, float )" )
	:EndDefine()

	PlayerActor_mt = LuaTinker.DefineClass( "GwScene.Cutscene.PlayerActor" )
		:base( "Player" )
	:EndDefine()

	NpcActor_mt = LuaTinker.DefineClass( "GwScene.Cutscene.NpcActor" )
		:base( "Character" )
	:EndDefine()

	MonsterActor_mt = LuaTinker.DefineClass( "GwScene.Cutscene.MonsterActor" )
		:base( "Character" )
	:EndDefine()

	CutsceneController_mt = LuaTinker.DefineClass( "CutsceneController" )
		:method( "PlayerActor CreatePlayer( String, float, float )" )
		:method( "MonsterActor CreateMonster( UInt32, String, float, float )" )
		:method( "NpcActor CreateNpc( UInt32, String, float, float )" )
		:method( "bool DestroyActorById( UInt32 )" )
		:method( "bool DestroyActor( Character )" )
	:EndDefine()

	Scene_mt = LuaTinker.DefineClass( "Scene" )
		:base( "BaseScene" )
		:method( "LocalPlayer GetLocalPlayer()" )
		:method( "void PrintMapInfo()" )
		:method( "void EnableCamera( bool )" )
		:method( "void ShowEntities( bool )" )
		:method( "void ShowLocalPlayer( bool )" )
		:method( "void PlayObjectAnimation( String, String )" )
		:method( "int DoCameraShake( int id )" )
		:prop( "CutsceneController cutsceneController" )
		:smethod( "CameraControl GetCameraControl()" )
		:method( "int RunScript( String fileName )" )
	:EndDefine()

	WorldState_mt = LuaTinker.DefineClass( "WorldState" )
		:sprop( "Scene activeScene" )
	:EndDefine()

	UIScene_mt = LuaTinker.DefineClass( "UIScene" )
		:base( "MonoBehaviour" )
	:EndDefine()

	UIManager_mt = LuaTinker.DefineClass( "UIManager" )
		:sprop( "UIManager sharedInstance" )
		:method( "UIScene FindScene( String )" )
		:method( "bool CloseScene( UIScene, bool play )" )
		:method( "void CloseAllScene( bool reset )" )
		:method( "void HideAll()" )
		:method( "void ShowAll()" )
		:method( "void HideAllExclude( String )" )
		:method( "void ShowAllExclude( String )" )
	:EndDefine()

	UnityUtils_mt = LuaTinker.DefineClass( "UnityUtils" )
		:smethod( "GameObject ResetTransform( GameObject )" )
		:smethod( "void CopyTransform( Transform, Transform )" )
		:smethod( "Transform Reset( Transform )" )
		:smethod( "Transform GetChildByName( Transform, String )" )
		:smethod( "Transform FindParent( Transform, Transform )" )
		:smethod( "String GetHierarchyPath( Transform )" )
		:smethod( "Transform Find( String )" )
		:smethod( "Transform RequireChildTransform( Transform, String )" )
		:smethod( "Transform SearchHierarchy( Transform, String, bool )" )
		:smethod( "Transform GetParentTrans( Transform, String )" )
	:EndDefine()

	ResLoader_mt = LuaTinker.DefineClass( "UniScene.ResLoader" )
		:smethod( "void AddModelPatnInfoFromLuaStr(string modelInfoStr) " )
		:smethod( " void PreLoadFromModelList() " )
	:EndDefine()

	MissionManager_mt = LuaTinker.DefineClass( "NewMission.MissionManager" )
		:sprop( "MissionManager Instance" )
		:prop( "MissionUI MissUI" )
	:EndDefine()

	MissionUI_mt = LuaTinker.DefineClass( "NewMission.MissionUI" )
		:method( "void OpenShowOnePanel( int showGroupId )" )
		:method( "void CloseCutScenePanel()" )
	:EndDefine()

	TraceMissionPanel_mt = LuaTinker.DefineClass( "NewMission.TraceMissionPanel" )
		:base( "UIScene" )
	:EndDefine()

	ArrowPanel_mt = LuaTinker.DefineClass( "ArrowPanel" )
		:method( "void Close()" )
	:EndDefine()

	MissHintEffect_mt = LuaTinker.DefineClass( "MissHintEffect" )
		:method( "void Close()" )
	:EndDefine()

	LuaTinker.DefineEnum( "ArrowPanel+ArrowDir" )
	--[[
		Left = 0,
        Down = 90,
        Right = 180,
        Up = 270
	--]]
	SinglePanelMgr_mt = LuaTinker.DefineClass( "SinglePanelMgr" )
		:sprop( "SinglePanelMgr sharedInstance" )
		:prop( "TraceMissionPanel MainTraceMissionPanel" )
		:method( "MissHintEffect SetMissHintEffect( Transform )" )
	:EndDefine()

	MemoryUtils_mt = LuaTinker.DefineClass( "MemoryUtils" )
		:smethod( "void SetRAMInterval( float f )" )
		:smethod( "void SetLowRAMStandard( float f )" )
		:smethod( "void SetMidRAMStandard( float f )" )
		:smethod( "void SetHighRAMStandard( float f )" )
	:EndDefine()

	GCUtils_mt = LuaTinker.DefineClass( "GCUtils" )
		:smethod( "void SetUnloadMininumInterval( float f )" )
		:smethod( "void SetGCMiniInterval( float f )" )
		:smethod( "void SetIsPrintMemory( bool b )" )
	:EndDefine()

	DevicesInfo_mt = LuaTinker.DefineClass( "PlatformHelper.DevicesInfo" )
		:smethod( "string GetPhoneBrand()" )
		:smethod( "string GetPhoneModel()" )
		:smethod( "string GetPhoneImei()" )
		:smethod( "string GetPhoneImsi()" )
		:smethod( "string GetPhoneNumber()" )
		:smethod( "string GetSimOperator()" )
		:smethod( "double GetPhoneMaxRAM()" )
		:smethod( "double GetAvavibleRAM()" )
		:smethod( "string GetPhoneCPUName()" )
		:smethod( "double GetPhoneCPUFrequency()" )
		:smethod( "int GetPhoneCores()" )
	:EndDefine()

	VFXPreLoader_mt = LuaTinker.DefineClass( "GwScene.VFXPreLoader" )
		:smethod( "void AsyncRequireLoad( string resourcesPath )" )
	:EndDefine()

	PlayerModelPathGroup_mt = LuaTinker.DefineClass( "GwScene.ModelPathMan+PlayerModelPathGroup" )
		:field( "String baseBodyPath" )
		:field( "String morphedBodyPath" )
	:EndDefine()

	ModelPathMan_mt = LuaTinker.DefineClass( "GwScene.ModelPathMan" )
		:smethod( "GwScene.ModelPathMan+PlayerModelPathGroup GetPlayerModelPathGroup( int index )" )
		:smethod( "void AsyncRequireLoadForLua( string resourcesPath )" )
	:EndDefine()

	ResourcesManager_mt = LuaTinker.DefineClass( "ResourcesManager" )
		:sprop( "ResourcesManager sharedInstance" )
		:method( "void AsyncLoadUIAssetForLua( string file )" )
	:EndDefine()

	Resolution_mt = LuaTinker.DefineClass( "UnityEngine.Resolution" )
		:prop( "int width" )
		:prop( "int height" )
	:EndDefine()

	Screen_mt = LuaTinker.DefineClass( "UnityEngine.Screen" )
		:sprop( "Resolution currentResolution" )
	:EndDefine()

	UIHelps_mt = LuaTinker.DefineClass( "Common.CommonHelper.UIHelps" )
		:smethod( "void SetUIRootContentSize( int width, int height )" )
	:EndDefine()

	QualitySettingsEx_mt = LuaTinker.DefineClass( "Common.CommonHelper.QualitySettingsEx" )
		:smethod( "void SetQualitySettings( string qualityname )" )
	:EndDefine()

	Application_mt = LuaTinker.DefineClass( "UnityEngine.Application" )
		:sprop( "bool isEditor" )
		:smethod( "void OpenURL( string url )" )
	:EndDefine()

	USCutscene_mt = LuaTinker.DefineClass( "GwScene.USCutscene" )
		:sprop( "USCutscene Instance" )
		:prop( "bool isLoading" )
		:prop( "string sourceFileName" )
		:prop( "bool isLoaded" )
		:prop( "bool isPlaying" )
		:method( "bool Load( string name )" )
		:method( "void Unload()" )
		:method( "void Play()" )
		:method( "void Stop()" )
		:method( "void OpenDialogBox( int id )" )
		:method( "void CloseDialogBox()" )
		:method( "void IdleAllActors" )
		:method( "Transform FindActor( String path )" )
	:EndDefine()

	Physics_mt = LuaTinker.DefineClass( "UnityEngine.Physics" )
		:sprop( "UnityEngine.Vector3 gravity" )
	:EndDefine()

	UIScene_mt = LuaTinker.DefineClass( "UIScene" )
		:base( "UnityEngine.MonoBehaviour" )
	:EndDefine()

	MainPanel_mt = LuaTinker.DefineClass( "MainPanel" )
		:base( "UIScene" )
	:EndDefine()

	Rigidbody_mt = LuaTinker.DefineClass( "UnityEngine.Rigidbody" )
		:base( "UnityEngine.Component" )
		:prop( "bool isKinematic" )
	:EndDefine()
end

return {
	Load = Load,
}
