local Root = "L:/Client"
local AreaStr = ""

if io and io.popen and not LuaTinker then
	local cd = io.popen( "cd" ):read( '*l' )
	print( "cd: "..cd )
	local s, e = string.find( cd, "Client" )
	if e then
		Root = string.sub( cd, 1, e )
	end
end

if arg and arg[1] and arg[2] then
	Root = arg[1]
	AreaStr = arg[2]
end

Root = string.gsub( Root, '\\', '/' )
local DatabaseRoot = Root.."/Assets/StreamingAssets/Database/"..AreaStr
local LuaRoot = Root.."/Assets/StreamingAssets/LuaRoot"

local DatabaseLocalTextName = "_LocaleText"
local StringBankOutput = DatabaseRoot.."/"..DatabaseLocalTextName..".lua"
local StringBankCSVOutput = DatabaseRoot.."/"..DatabaseLocalTextName..".csv"
local StringInCodeSourcePath = DatabaseRoot.."/../../../Plugins/Client/Misc/Localization/TextDef.cs"
local StringHelpSourcePath = Root.."/Assets/ToolOutput/Help"
local StringHelpFlsPath = Root.."/Assets/StreamingAssets/FileRoot/Help.fls"
local MaxStringBankBinSize = 524288
local LanguageLeadingTag = '@'
local StringInCode = "strings"
local DatabaseTool = require "DatabaseTool"

local AllSupportedLanguages = {
	["chs"] = true, -- chinese simplify
	["cht"] = true, -- chinese tranditional
	["kor"] = true, -- korean
	["eng"] = true,  -- english
}

local FileExcludes = {
	[DatabaseLocalTextName] = true,
	Chinese = true,
	Korean = true,
	English = true,
	Traditional = true,
	offstringlogin = true,
	offstringingame = true,
	modelInfo = true,
	RoleAnimEventBank = true,
}

local AllLocalizeFiles = {
	[StringInCode] = true
}

local function scanAllFileNames( path )
	path, _ = path:gsub( "/", "\\" )
	local ret = {}
	for dir in io.popen( string.format( "dir \"%s\" /S/b", path ) ):lines() do
		local s, e, f = dir:find( ".+\\(.+)%.lua$" )
		if f then
			table.insert( ret, f )
		end
	end
	return ret
end

local function scanAllTxtFileNames( path )
	path, _ = path:gsub( "/", "\\" )
	local ret = {}
	for dir in io.popen( string.format( "dir \"%s\" /S/b", path ) ):lines() do
		local s, e, f =dir:find("(.+)%.txt$")
		if f then
			table.insert( ret, dir )
		end

		--local s, e, f , g=dir:find("(.+)Client(.+)%.txt$")
		--if g then
		--	table.insert( ret, g )
		--	print(g)
		--end
	end
	return ret
end

local function hashString( v )
	local val = 0
	for i = 1, #v do
		local c = string.byte( string.sub( v, i, i ), 1 )
		val = val + c * 193951
		val = math.fmod( val, MaxStringBankBinSize )
		val = val * 399283
		val = math.fmod( val, MaxStringBankBinSize )
	end
	return val
end

local function EncodeEscapeString( s )
	local buf = {}
	buf[#buf + 1] = "\""
	string.gsub( s, ".",
		function ( c )
			if c == '\n' then
				buf[#buf + 1] = "\\n"
			elseif c == '\t' then
				buf[#buf + 1] = "\\t"
			elseif c == '\r' then
				buf[#buf + 1] = "\\r"
			elseif c == '\a' then
				buf[#buf + 1] = "\\a"
			elseif c == '\b' then
				buf[#buf + 1] = "\\b"
			elseif c == '\\' then
				buf[#buf + 1] = "\\\\"
			elseif c == '\"' then
				buf[#buf + 1] = "\\\""
			elseif c == '\'' then
				buf[#buf + 1] = "\\\'"
			elseif c == '\v' then
				buf[#buf + 1] = "\\\v"
			elseif c == '\f' then
				buf[#buf + 1] = "\\\f"
			else
				buf[#buf + 1] = c
			end
		end
	)
	buf[#buf + 1] = "\""
	return table.concat( buf, "" )
end

local function addStringToBank( stringBank, str )
	local hash = hashString( str )
	local _v = stringBank[hash]
	while _v do
		if _v == str then
			print( "" )
			return hash
		end
		hash = hash + 1
		hash = math.mod( hash, MaxStringBankBinSize )
		_v = stringBank[hash]
	end
	stringBank[hash] = str
	return hash
end

local function dumpStringBank( stringBank )
	print( 'dump database local string bank begin...' )
	for k, v in pairs( stringBank ) do
		print( string.format( "\t[%g] = %s", k, v ) )
	end
	print( 'dump database local string bank end.' )
end

local function saveStringBank( stringBank, tofile )
	if tofile then
		local fileName = StringBankOutput
		local _file, err = io.open( fileName )
		if _file ~= nil then
			_file:close()
			os.remove( fileName )
		end
		file = io.open( fileName, "w" )
		file:write( string.format( "local %s = {\n", DatabaseLocalTextName ) )
		for k, v in pairs( stringBank ) do
			file:write( string.format( "\t[%g] = %s,\n", k, EncodeEscapeString( v ) ) )
		end
		file:write( "}\n" )
		file:write( string.format( "return %s\n--EOF", DatabaseLocalTextName ) )
		file:close()
	else
		dumpStringBank( stringBank )
	end
end

local function saveStringBankCSV( stringBank, tofile )
	if tofile then
		local csv = require "std.csv"
		local fileName = StringBankCSVOutput
		local _file, err = io.open( fileName )
		if _file ~= nil then
			_file:close()
			os.remove( fileName )
		end
		local t = {}
		local count = 1
		for k, v in pairs( stringBank ) do
			t[count] = { k, v }
			count = count + 1
		end
		table.sort( t,
			function( a, b )
				return a[1] < b[1]
			end
		)
		csv.save( fileName, t, true )
	else
		dumpStringBank( stringBank )
	end
end

local function loadStringBankCSV()
	local stringBank = {}
	local csv = require "std.csv"
	local fileName = StringBankCSVOutput
	local file, err = io.open( fileName )
	if file then
		print( "load string bank: "..fileName )
		file:close()
		local b = csv.load( fileName, true )
		for i = 1, #b do
			stringBank[b[i][1]] = b[i][2]
		end
	end
	return stringBank
end

local function createStringBank( info )
	local stringBank = {}
	setmetatable( stringBank, {} )
	local chunk = loadfile( StringBankOutput )
	if chunk then
		print( 'load string bank: '..StringBankOutput )
		local last = chunk()
		if last and type( last ) == "table" then
			for k, v in pairs( last ) do
				stringBank[k] = v
			end
		end
	end
	return stringBank
end

local function checkIfNeedUtf8( v )
	if v ~= nil and type( v ) == "string" then
		for i = 1, #v do
			local c = string.byte( string.sub( v, i, i ), 1 )
			if c < 0 or c > 127 then
				utf8 = true
				return true
			end
		end
	end
	return false
end

local function LocalizeDatabase( locale, tofile )
	if locale and type( locale ) == "string" and #locale > 1 then
		local xstr = require "std.xstr"
		locale = xstr.lower( locale )
		local valid = false
		local excludes = {}
		for v, _ in pairs( AllSupportedLanguages ) do
			if v == locale then
				valid = true
			else
				local x = string.format( "@%%a+.*_(%s)$", v )
				excludes[#excludes + 1] = x
			end
		end
		if not valid then
			print( "invalid language type: "..locale )
			return
		end

		print( "current language: "..locale )
		local partten = string.format( "@%%a+.*_(%s)$", locale )
		local files = scanAllFileNames( DatabaseRoot )
		for _, v in ipairs( files ) do
			if not FileExcludes[v] then
				DatabaseTool.Reload( v )
				local t = Database[v]
				if t then
					DatabaseTool.TrimDatasetFile(
						t, excludes,
						tofile
					)
				end
			end
		end
	end
end


-- helptxt(޸)Ӧid
local function AddStrinfIdToHelpTxt( path, strBank, tofile )
	local f = io.open( path )
	local size = f:seek( "end" )
	local data = ""
	if size >= 3 then
		f:seek( "set", 0 )
		local _1 = string.byte( f:read( 1 ) )
		local _2 = string.byte( f:read( 1 ) )
		local _3 = string.byte( f:read( 1 ) )
		if _1 ~= 0xef or _2 ~= 0xbb or _3 ~= 0xbf then
			f:seek( "set", 0 )
			data = f:read( "*a" )
		else
			f:seek( "set", 3 )
			data = f:read( "*a" )
		end
	else
			f:seek( "set", 0 )
			data = f:read( "*a" )
	end
	f:close()

	local id = addStringToBank( strBank,  data )
	if tofile then
		local fn = path:gsub( "ToolOutput" , "StreamingAssets" )
		local file = io.open( fn, "w" )
		if not file then
			local a, b, c = fn:find( "(.+)\\.+.txt" )
			if c then
				os.execute( string.format( "mkdir \"%s\"", c ) )
				file = assert( io.open(  fn, "w" ) )
			end
		end
		if file then
			file:write( id )
			file:close()
		end
	else
		print( string.format( "help: %s -> %d", path, id ) )
	end
end

local function localizeRecord( id, record, genCode, StringBank )
	local localized_fields = nil
	local subTable = nil
	for k, v in pairs( record ) do
		local vt = type( v )
		if vt == "string" then
			if localize_all or checkIfNeedUtf8( v ) then
				if #v > 0 and string.sub( v, 1, 1 ) == LanguageLeadingTag then
					print( string.format( "invalid leading character for localized text! key, value: %s, %s", k, v ) )
					os.execute( "pause" )
				end
				if not localized_fields then
					localized_fields = {}
				end
				-- build localized id string with tag
				local sid = addStringToBank( StringBank, v )
				localized_fields[k] = string.format( "%s%g", LanguageLeadingTag, sid )
				if genCode then
					genCode[#genCode + 1] = {
						id,
						sid,
						v
					}
				end
			end
		elseif vt == "table" then
			if not subTable then
				subTable = {}
			end
			subTable[#subTable + 1] = v
		end
	end
	local localized = false
	if localized_fields then
		-- override localized string with tag
		localized = true
		for k, v in pairs( localized_fields ) do
			record[k] = localized_fields[k]
		end
	end
	if subTable then
		for _, sub in ipairs( subTable ) do
			localized = localizeRecord( 0, sub, genCode, StringBank ) or localized
		end
	end
	return localized
end

local function ExportDatabaseLocalText( tofile, new )
	local StringBank = nil
	if new then
		StringBank = createStringBank()
	else
		StringBank = loadStringBankCSV()
	end
	local localized_dirty = false
	local files = scanAllFileNames( DatabaseRoot )
	for _, v in ipairs( files ) do
		if not FileExcludes[v] then
			local localize_all = AllLocalizeFiles[v]
			local genCode = nil
			if StringInCode == v then
				genCode = {}
			end
			print( "DatabaseTool.Load :"..v )
			local t = DatabaseTool.Load( v )
			if t then
				local localized = false
				for id, record in pairs( t ) do
					if type( record ) == "table" then
						localized = localizeRecord( id, record, genCode, StringBank )
					end
				end
				if localized then
					localized_dirty = true
					DatabaseTool.TrimDatasetFile(
						Database[v], {},
						tofile
					)
				end
			end
			if genCode then
				if not tofile then
					print( "dump string in code:" )
					print( "-----------------------------------------------------" )
					for _, v in ipairs( genCode ) do
						print( string.format( "%s, %d, %s", v[1], v[2], v[3] ) )
					end
					print( "-----------------------------------------------------" )
				else
					local fn = StringInCodeSourcePath
					local _f = io.open( fn, "rb" )
					if _f then
						_f:close()
						print( "remove file: "..fn )
						os.remove( fn )
					end
					local f = io.open( fn, "wb" )
					f:write( string.char( 0xef ) )
					f:write( string.char( 0xbb ) )
					f:write( string.char( 0xbf ) )
					f:write( "//This file generated by LocalizeDatabase.lua, if you have any questions please contant to lujian.\n" )
					f:write( "namespace Locale {\n" )
					f:write( "\tpublic enum TextDef {\n" )
					for _, v in ipairs( genCode ) do
						f:write( string.format( "\t\t%s = %d,//%s\n", v[1], v[2], EncodeEscapeString( v[3] ) ) )
					end
					f:write( "\t}\n" )
					f:write( "}\n" )
					f:close()
				end
			end
		end
	end

	local helpFiles = scanAllTxtFileNames( StringHelpSourcePath )
	local flsResult = "100, 100000"
	local helpRootLength = string.len(StringHelpSourcePath) + 2	-- ȥ L:\Client\Assets\ToolOutput\Help\
	for _, v in ipairs( helpFiles ) do
		AddStrinfIdToHelpTxt( v, StringBank, tofile )
		local helpFls = string.sub(v,helpRootLength, -1):gsub( "\\", "/" )
		flsResult = flsResult.."\n"..helpFls
	end
	local file = io.open( StringHelpFlsPath, "w"  )
	file:write(flsResult)

	if localized_dirty then
		saveStringBankCSV( StringBank, tofile )
	else
		print( "\nThere is no locale text be created.\n" )
	end
	print( "ExportDatabaseLocalText done." )
end

return {
	LocalizeDatabase = LocalizeDatabase,
	ExportDatabaseLocalText = ExportDatabaseLocalText,
	IsLanguageSupported = function( name )
		return AllSupportedLanguages[name] == true
	end,
}


